%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Bot, J, Wessels, L, Reinders, M.
%                     Co-occurrence analysis of insertional
%                     mutagenesis data reveals cooperating
%                     oncogenes. Bioinformatics, 2007, 
%                     23(13), i133-i141
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Function to compute the peaks in the cooccurrence space

%INPUTS:
%in             - 2D input data (ICs)
%h              - scale parameter, width of the kernel function
%options        - options struct used for optimizer
%binsize        - optional, binsize in multiples of h
%OUTPUTS:
%peak_pos       - positions of the peaks
%peak_height    - height of the peaks
%no_unity_peaks - Number of peaks of height one

function [peak_pos, peak_height, no_unity_peaks] = parzen2Dpeak(in,h,options,binsize)
h = [h,h];
NN=length(in);
if isempty(binsize)
    binsize = 5; %Relative to h
end

%Bin the data using h in 2D, data_index gives the binindex per datapoint
data_index = ceil([in(:,1)/(binsize*h(1)) in(:,2)/(binsize*h(2))]);
%Number of bins
nobins = max(max(data_index));
%Indexed data index
%data_I = sub2ind([nobins nobins],data_index(:,1),data_index(:,2));
%Find nonzero bins and points counts
[non_zero_bins,dummy,point_cnt] = unique(data_index,'rows');
point_cnt = histc(point_cnt,1:size(non_zero_bins,1));
%Make a sparse matrix, but make it a little bigger
binmat = sparse(nobins+2,nobins+2);
%Set the nonzero bins to 1, but remember the first column and row are zero
II = sub2ind(size(binmat),non_zero_bins(:,1)+1,non_zero_bins(:,2)+1);
binmat(II)=point_cnt;
%Look for interesting bins: check the surrounding bins OR
% have multiple points in one bin
andmat =    ...
    binmat(2:end-1,2:end-1)>1|... Bins with multiple points
    binmat(2:end-1,2:end-1)&... 5 Nonzero bins
    (binmat(1:end-2,1:end-2)|... 1 Top left
    binmat(2:end-1,1:end-2)|... 2 Top middle
    binmat(3:end  ,1:end-2)|... 3 Top right
    binmat(1:end-2,2:end-1)|... 4 Middle left
    binmat(3:end  ,2:end-1)|... 6 Middle Right
    binmat(1:end-2,3:end  )|... 7 Bottom left
    binmat(2:end-1,3:end  )|... 8 Bottom middle
    binmat(3:end  ,3:end  )); %  9 Bottom right
%Get the isolated points (no need to include these)
unity_bins = (~andmat)&binmat(2:end-1,2:end-1);

no_unity_peaks = length(find(unity_bins==1));
%Compute the indices of the interesting bins
I = find(andmat);
[n,m] = ind2sub(size(andmat),I);
interesting_bins = [n m];
%Total number of bins to check
N = length(I);

%precompute constants that only depend on h
h1s = h(1)^2;
h2s = h(2)^2;
denom = (2*(h(1)*h(2))^2);

%loop over all datapoints
%reserve some space
res = zeros(NN+1,2);
fval = zeros(NN+1,1);
cnt = 2;
for ii = 1:N
    n = interesting_bins(ii,1);
    m = interesting_bins(ii,2);
    roibins = ...
        [data_index(:,1)==n+1&data_index(:,2)==m+1|...
        data_index(:,1)==n  &data_index(:,2)==m+1|...
        data_index(:,1)==n-1&data_index(:,2)==m+1|...
        data_index(:,1)==n+1&data_index(:,2)==m  |...
        data_index(:,1)==n  &data_index(:,2)==m  |...
        data_index(:,1)==n-1&data_index(:,2)==m  |...
        data_index(:,1)==n+1&data_index(:,2)==m-1|...
        data_index(:,1)==n  &data_index(:,2)==m-1|...
        data_index(:,1)==n-1&data_index(:,2)==m-1];

    %Find data in bin neighborhood
    roidata = in(roibins,:);
    K = size(roidata,1);
    %Find data in bin
    bindata = in(data_index(:,1)==n  &data_index(:,2)==m,:);
    for j = 1:size(bindata,1)
        x0 = bindata(j,:);
        [res(cnt,:),fval(cnt)] = fminunc(@(x)parz2Doptim(x,roidata,K,h1s,h2s,denom,1),x0,options);
        %Check if this overlaps with another peak already found. This
        %occurs when x and y differences are smaller than 0.01*sclae
        if j~=1 ...
             && any(abs(res(cnt,1)-res(1:cnt-1,1))<0.01*h(1) ...
                   & abs(res(cnt,2)-res(1:cnt-1,2))<0.01*h(1))
            fval(cnt) = 0;
        end
        cnt = cnt+1;
    end
end
peak_pos = res(fval~=0,:);
peak_height = -fval(fval~=0);

function [f,g] = parz2Doptim(x,in,K,h1s,h2s,denom,C)
f_arg = zeros(1,K);
d_arg1 = zeros(1,K);
d_arg2 = zeros(1,K);
for i = 1:K
    x1 = in(i,1);
    x2 = in(i,2);
    f_arg(i) = exp(-(h2s*((x(1)-x1)^2)+h1s*((x(2)-x2)^2))/denom);
    d_arg1(i) = -(x(1)-x1);
    d_arg2(i) = -(x(2)-x2);
end
f = -C*sum(f_arg);
g(1) = -(C/h1s)*sum(d_arg1.*f_arg);
g(2) = -(C/h2s)*sum(d_arg2.*f_arg);